/**
 * program to align brakpoint regions using partially local alignment.
 * @author: M. AL Arab
 */

#include <iostream>
#include <fstream>
#include <string>
#include <vector>
#include <algorithm>
#include <iomanip>
#include <sstream>
#include <math.h>
#include <unistd.h>
#include <typeinfo>
#include <cstdio>
#include <cctype>
#include <stdio.h>      /* printf, fgets */
#include <stdlib.h>
#include <cmath>
#include <typeinfo>
#include <algorithm> // for std::find
#include <iterator> // for std::begin, std::end
#include <map>
#include <boost/algorithm/string.hpp>

using namespace std;
/*
 * return maximum of a vector of 5 entries and the which of the 5 (first,2nd,3rd...)
 */
std::pair<float,int> max2d(vector <float> v, int * ptr, int maxj) {
//	int max = -10000;
	vector <int> ind(0);
	int index;
	float max = *max_element(v.begin(), v.end());
	for (int i = 0; i<v.size(); i++)
	{
		if (v[i]==max)
		{
//			index = i;

			ind.push_back(i);
		}
	}
	int rnd = rand()%ind.size();
	index = ind[rnd];
	*ptr=-1;
	if (index==0) {

		*ptr = 0;
	} else if (index==1) {
		*ptr = 1;
	} else if (index==2) {
		*ptr = 2;
	}
	else if (index==3){
		*ptr = -2;//gap =>S_imax,jmax,0

	}
//	else if (index ==4){
//		*ptr = -1;
//	}
	else if (index==4){
		*ptr =11;//overlap s[i][maxj][k]
//		cout<<v[3]<<" v[4] "<<v[4]<<endl;
	}


	return std::make_pair(max,maxj);
}

/*
 * Returns the maximum of a vector of 8 entries and which one is the maximum (ptr).
 */
double max3d(vector <float> v, int *ptr)
{
vector <int> ind(0);
int index;
float max = *max_element(v.begin(), v.end());
for (int i = 0; i<v.size(); i++)
{
	if (v[i]==max)
	{
		ind.push_back(i);
	}
}

int rnd = rand()%ind.size();
index = ind[rnd];
*ptr=-1;
 if (index ==0)
 {
	 *ptr = 3;
 }
 else if (index ==1)
 {
	 *ptr = 4;
 }
 else if (index ==2)
  {
 	*ptr = 5;
  }
 else if (index ==3)
  {
	 *ptr = 6;
  }
 else if (index ==4)
  {
	 *ptr = 7;
  }
 else  if (index ==5)
  {
	 *ptr = 8;
  }
 else if (index ==6)
  {
	 *ptr = 9;
  }
 else if (index ==7)
 {
	 *ptr =10;
 }

return max;

}

/*
 * scoring function for the 2d matrix 
 * @param m match score
 * @param u mismatch score
 * @param gap gap score
 * @param nuc1 first nucleotide 
 * @param nuc2 second nucleotide
 */
double delta(double m, double u, double gap, char nuc1, char nuc2) {
	if ((nuc1 == '-') && (nuc2 == '-')) {
		return 0;

	}

	else if ((nuc1 == '-') || (nuc2 == '-')) {
		return gap;
	} else {
		return (nuc1 == nuc2) ? m : u;
	}
}
/*
 * scoring function for the 3d matrix 
 * @param sp sum of pairs weight
 * @param m match score
 * @param u mismatch score
 * @param gap gap score
 * @param nuc1 first nucleotide 
 * @param nuc2 second nucleotide
 * @param nuc3 third nucleotide
 */
double gama(float sp,double m, double u, double gap, char n1, char n2, char n3) {

	return (delta(m, u, gap, n1, n2) + delta(m, u, gap, n1, n3)
			+ delta(m, u, gap, n2, n3))/sp ;

}


/*
 * check if file is empty
 */
bool is_empty_check(std::ifstream& pFile)
{
    return pFile.peek() == std::ifstream::traits_type::eof();
}

/*
 * convert string to lower case
 */
void convert_tolower(string & s)
{
  std::transform(s.begin(), s.end(), s.begin(), ::tolower);

}
void convert_toupper(string & s)
{
  std::transform(s.begin(), s.end(), s.begin(), ::toupper);

}

/*
 * read input fasta file with 3 sequences ref, left and right
 * 
 */
int read_infile(string name,string &ref, string &left, string &right) {
	std::string line;
	string token;
	int wcount = 0;
	ifstream fs(name.c_str());
	vector<string> seq;
	if (!fs) {
		std::cerr << "Couldn't open the input fasta file!"<<name.c_str();
		exit(1);
	}
	if (is_empty_check(fs))
	{
		std::cerr <<" The input fasta file is empty! "<<endl;
		exit(1);
	}
	while (fs>>token)
	{
	    if  (token.find(">") == 0)
	    {
	    wcount++;
	    }
	}
	fs.clear();
	fs.seekg(0, ios::beg);
	if (wcount != 3)
	{
		std:cerr << "The file must contains 3 sequences it contains "<<wcount<<" !"<<endl;
		exit(1);
	}
	string id;
	string content;
	while (getline(fs, line)) {
		 if( line.empty() || line[0] == '>' ){ // Identifier marker
		            if( !id.empty() ){ // Print out what we read from the last entry
		                id.clear();
		            }
		            if( !line.empty() ){
		                id = line.substr(1);
		            }
		            if (!content.empty())
		            seq.push_back(content);
		            content.clear();
		        } else if( !id.empty() ){
		            if( line.find(' ') != std::string::npos ){ // Invalid sequence--no spaces allowed
		                id.clear();
		                content.clear();
		            } else {
		                content += line;
		            }
		        }
}
	 if( !id.empty() ){ // Print out what we read from the last entry
		 if (!content.empty())
		 seq.push_back(content);
	    }
	 convert_toupper(seq[0]);
	 convert_toupper(seq[1]);
	 convert_toupper(seq[2]);
	 ref = seq[0];
	 left = seq[1];
	 right = seq[2];

	 return 0;
}

/*
 * write oupt alignment to a fasta file or stockholm file
 */
int write_output(string name, string input,string format)
{
        if (format == "fas" || format =="")
        {
        	std::ofstream out(name);
			out << input;
			out.close();
        }
        else if (format == "sto" )
        {
        	system(("python tosto.py  '"+input+"' "+name).c_str());
        }
        else 
        {
        	cerr<<"invalid format sto or fas!"<<endl;
        	exit(1);
        }
        	
	  return 0;
	}


/**
 * check if two floats are equal
 */
bool AreSame(double a, double b)
{
     return fabs(a - b) <=  0.00001 ;
}

/**
 * initialiase the 3 dimensional matrix as described in the paper.
 * @param ptr pointer to the source of the value in the cell for the trace back.
 * @param s 3D vector for the dynamic programming.
 * @param tb 3D vector for the traceback.
 * @param m length of first sequence.
 * @param n length of second sequence.
 * @param p length of third sequence.
 * @param a first sequence.
 * @param b second sequence.
 * @param c third sequence.
 * @param match match score.
 * @param unmatch mismatch score.
 * @param gap gap score.
 * @param sp sum of pairs weight.
 */
void  s_init(int &ptr, vector < vector < vector <float> > > &s, vector < vector < vector <int> > > &tb, int m, int n,int p,string a, string b, string c,double match, double unmatch , double gap,float sp)
{
	for (int i =0; i<=m; i++)
	{
		s[i][0][0]=gap*i;
		tb[i][0][0]=-1;
	}
	for (int j =0; j<=n; j++)
	{
		s[0][j][0]=gap*j;
		tb[0][j][0]=-1;
	}
	for (int k =0; k<=p; k++)
	{
		s[0][0][k]=gap*2*k;
		tb[0][0][k]=-1;
	}
	for (int i= 1; i<=m ; i++)
	{
		for (int j = 1 ; j<=n; j++)
		{
			vector <float>  v(0);
			v.push_back(s[ i-1 ][ j-1 ][0] +delta(match, unmatch , gap,a [i-1],b [j-1]));
			v.push_back( s[ i-1 ][ j ][0]   + delta(match, unmatch , gap,a [i-1],'-'));
			v.push_back( s[ i ][ j-1 ][0]   + delta(match, unmatch , gap,'-',b [j-1]));
			s[i][j][0]=max2d(v,&ptr,-1).first;
            tb [i][j][0]=ptr;
		}
	}
	for (int i =1; i<=m; i++)
	{
		for (int k =1; k<=p; k++)
		{
			vector <float>  v(0);
			v.push_back(s[i-1][0][k-1] + gama(sp,match, unmatch , gap,a [ i-1 ],'-',c [ k-1 ]));
			v.push_back(s[i-1][0][k]   + gama(sp,match, unmatch , gap,a [ i-1 ],'-', '-'));
			v.push_back( s[i][0][k-1]   + gama(sp,match, unmatch , gap,'-','-',  c [ k-1 ]));
			s[i][0][k] = max2d(v, &ptr,-1).first;
			 tb[i][0][k] =ptr;
		}
	}
//           //3rd plane;
	for (int j=1; j<=n; j++)
		{
			for (int k=1; k<=p; k++)
			{
				vector <float>  v(0);
				v.push_back(s[0][j-1][k-1] + gama(sp,match, unmatch , gap,b [j-1],c [k-1],'-'));
				v.push_back(s[0][j-1][k]   + gama(sp,match, unmatch , gap,b [j-1],'-','-'));
				v.push_back(s[0][j][k-1]   + gama(sp,match, unmatch , gap,'-',c [k-1],'-'));
			    s[0][j][k]= max2d(v,&ptr,-1).first;
				tb[0][j][k]=ptr;
			}
		}
 }

/**
 * fill the 3 dimensional matrix as described in the paper.
 * @param ptr pointer to the source of the value in the cell for the trace back.
 * @param s 3D vector for the dynamic programming.
 * @param tb 3D vector for the traceback.
 * @param m length of first sequence.
 * @param n length of second sequence.
 * @param p length of third sequence.
 * @param a first sequence.
 * @param b second sequence.
 * @param c third sequence.
 * @param match match score.
 * @param unmatch mismatch score.
 * @param gap gap score.
 * @param sp sum of pairs weight.
 */
void  s_fill(int &ptr,vector < vector < vector <float> > > &s,vector < vector < vector <int> > >  &tb,int m, int n,int p,string a, string b, string c,double match, double unmatch , double gap, float sp)
{
	for (int i =1; i<=m; i++)
		{
			for (int j = 1; j<=n; j++)
                {
                    for (int k =1; k<=p; k++)
                    {
                    	vector <float>  v(0);
                    	 v.push_back(s[i-1][j-1][k-1]  + gama(sp,match, unmatch , gap,a [i-1],b [j-1],c [k-1]));
                    	 v.push_back(s[i-1][j-1][k]   + gama(sp,match, unmatch , gap,a [i-1],b [j-1], '-'));
                    	 v.push_back(s[i-1][j][k-1]   + gama(sp,match, unmatch , gap,a [i-1],'-', c [k-1]));
                    	 v.push_back(s[i-1][j][k]     + gama(sp,match, unmatch , gap,a [i-1],'-','-'));
                    	 v.push_back(s[i][j-1][k-1]   + gama(sp,match, unmatch , gap,'-',b [j-1], c [k-1]));
                    	 v.push_back(s[i][j-1][k]     + gama(sp,match, unmatch , gap,'-',b [j-1],'-'));
                    	 v.push_back(s[i][j][k-1]     + gama(sp,match, unmatch , gap,'-','-',c [k-1]));
                    	 v.push_back(s[i][j][0]);
                    	 s[i][j][k] = max3d(v,&ptr);
                    	 tb[i][j][k] = ptr;
//                    	}
                    }
                }
            }
        }

std::pair<int,int> max_2ind(vector<vector <vector <float> > > &s,int m,int n)
{
	 float max = -1*std::numeric_limits<float>::max();
	 int imax;
	 int jmax;
	 for (int i =0; i<=m; i++)
	 {
		 for (int j =0; j<=n; j++)
		 {
			 if (s[i][j][0]>=max)
			 {
				 max = s[i][j][0];
				 imax = i;
				 jmax =j;
			 }
		 }
	 }
	 return std::make_pair(imax,jmax);
}

 float max_ind(vector<vector <vector <float> > > &s,int i, int k,int n)
 {
	 float max = -1*std::numeric_limits<float>::max();
	 int maxj ;
	 for (int j = 0; j<=n; j++)
	 {
		 if (s[i][j][k]>max)
		 {
			 max = s[i][j][k];
			 maxj=j;
		 }
	 }
	 return maxj;
 }

 /**
  * fill the 2 dimensional matrix as described in the paper.
  * @param ptr pointer to the source of the value in the cell for the trace back.
  * @param s2 2D vector for the dynamic programming.
  * @param tb2 2D vector for the traceback.
  * @param m length of first sequence.
  * @param p length of third sequence.
  * @param a first sequence.
  * @param c third sequence.
  * @param match match score.
  * @param unmatch mismatch score.
  * @param gap gap score.
  * @param sp sum of pairs weight.
  */
	void s2_fill(int &ptr,vector <vector <float> > &s2, vector < vector < vector <float> > > &s,vector < vector <int>  >  &tb2,int m, int n,int p,string a, string b, string c,double match, double unmatch , double gap, float sp, int &maxj)
	{
		for (int i =0; i<m; i++)
		{
			s2[i][0]=0;
			tb2[i][0]=-1;
		}
		for (int k=0; k<p ; k++)
		{
			s2[0][k]=k*0;
			tb2[0][k]=-1;
		}

		for (int i =1; i<=m; i++)
			{
				for (int k =1; k<=p; k++)
				{
					vector <float>  v(0);
					maxj = max_ind(s,i,k,n);
					int imax = max_2ind(s,i,n).first;
					int jmax = max_2ind(s,i,n).second;
					v.push_back(s2[i-1][k-1] + delta(match, unmatch , gap,a [ i-1 ],c [ k-1 ]));
					v.push_back(s2[i-1][k]   + delta(match, unmatch , gap,a [ i-1 ],'-'));
					v.push_back(s2[i][k-1]   + delta(match, unmatch , gap,'-',c [ k-1 ]));
					v.push_back(s[imax][jmax][0]);//gap
					v.push_back(s[i][maxj][k]);//overlap
					s2[i][k] = max2d(v,&ptr,maxj).first;
					 tb2[i][k]=ptr;
				}
			}
	}


double find_max(double a) { return a; } //  base case for recursion
template<typename T>
double find_max(const std::vector<T>& v)
{
   double r = find_max(v.front());
   for (auto i=v.begin()+1, e=v.end(); i<e; ++i)
   {
      double c = find_max(*i);
      if (c>r) r=c;
   }
   return r;
}

/**
 * trace back the 3D matrix.
 * @param &s 3D vector for the dynamic programming.
 * @param &tb 3D vector for the traceback.
 * @param m length of first sequence.
 * @param n length of second sequence.
 * @param p length of third sequence.
 * @param a first sequence.
 * @param b second sequence.
 * @param c third sequence.
 * @param al_a the alignment record of the first sequence
 * @param al_b the alignment record of the second sequence
 * @param al_c the alignment record of the third sequence
 * @param match match score.
 * @param unmatch mismatch score.
 * @param gap gap score.
 * @param sp sum of pairs weight.
 * @param ismax?
 * @param maxi index at the first sequence which contains the highest score.
 * @param maxj index at the second sequence which contains the highest score.
 * @param maxk index at the third sequence which contains the highest score.
 */
void s_back(vector <vector <vector <float> > > &s ,vector < vector <vector <int> > > &tb, string a, string b, string c, string &al_a,string &al_b, string &al_c, double match, double unmatch ,double gap, int maxj, int maxi,int maxk)
{

			int i=maxi;
			int j = maxj;
			int k = maxk;
			   while (i> 0 && j>0 && k>0)
			   {
				   switch (tb[i][j][k])
				   {
				   case 3:
					  al_a += a [i-1];
					  al_b += b [j-1];
					  al_c += c [k-1];
					  i--; j--; k--;
					  break;
				   case 4:
					  al_a += a [i-1];
					  al_b += b [j-1];
					  al_c += '-';
					  i--;j--;
					  break;
				   case 5:
					   al_a += a [i-1];
					  al_b += '-';
					  al_c += c [k-1];
					  i--; k--;
					  break;
				   case 6:
					   al_a += a [i-1];
					   al_b += '-';
					   al_c += '-';
					   i--;
					   break;
				   case 7:
					   al_a += '-';
					   al_b += b [j-1];
					   al_c += c [k-1];
					   j--; k--;
					   break;

				   case 8:
						 al_a += '-';
						 al_b += b [j-1];
						 al_c += '-';
						 j--;
					 break;
				   case 9:
					   al_a += '-';
					   al_b += '-';
					   al_c += c [k-1];
					   k--;
					   break;

				   case 10:
					   while (i>0 && j>0 )
						{
					   switch (tb[i][j][0])
						   {
						   case 0:
							  al_a += a[i-1];
							  al_b += b[j-1];
							  al_c += '-';
							  i--; j--;
							  break;

						   case 1:
							  al_a += a[i-1];
							  al_b += '-';
							  al_c += '-';
							  i--;
							   break;

						   case 2:
							  al_a += '-';
							  al_b += b[j-1];
							  al_c += '-';
							  j--;
							 break;
						   }
					   }
					   break;
				   default:
					   cout<<"problem default"<<endl;
				   }
			   }
//				   cout<< " I J K "<<i << " "<<j << " "<<k <<endl ;
			   if (i>0 && j>0)
			   {
//					   cout<<"#Overlap="<<maxj-j<<endl;
			   }
			   while (i>0 && j>0 )
				{
			   switch (tb[i][j][k])
				   {
				   case 0:
					  al_a += a[i-1];
					  al_b += b[j-1];
					  al_c += '-';
					  i--; j--;
					  break;
				   case 1:
					  al_a += a[i-1];
					  al_b += '-';
					  al_c += '-';
					  i--;
					   break;
				   case 2:
					  al_a += '-';
					  al_b += b[j-1];
					  al_c += '-';
					  j--;
					 break;
				   }
			   }
			while(i>0)
			{
				al_a += a[i-1];
				al_b += '-';
				al_c +='-';
				i--;
			}
			while(j>0)
			{
				al_a += '-';
				al_b += b [j-1];
				al_c +='-';
				j--;
			}
}
/**
 * trace back the 2D matrix.
 * @param &s2 2D vector for the dynamic programming.
 * @param &tb2 2D vector for the traceback.
 * @param m length of first sequence.
 * @param p length of third sequence.
 * @param a first sequence.
 * @param c third sequence.
 * @param al_a the alignment record of the first sequence
 * @param al_b the alignment record of the second sequence
 * @param al_c the alignment record of the third sequence
 * @param match match score.
 * @param unmatch mismatch score.
 * @param gap gap score.
 * @param sp sum of pairs weight.
 * @param ismax?
 * @param maxj (nit used)
 
 */
void s2_back(vector <vector <vector <float> > > &s, vector <vector <float> > &s2 ,vector < vector <int> >  &tb2,vector < vector <vector <int> > > &tb, string a, string b, string c, string &al_a,string &al_b, string &al_c, double match, double unmatch ,double gap, int &maxj)

{
	int i=a.length();
	int k = c.length();
	int imax;
	int jmax;
    int count =0;
	bool out = false;
	while(i> 0 && k>0)
	{
	 switch (tb2[i][k])
	  {

	 case 0:
		 al_a += a [i-1];
		 al_b += '-';
		 al_c += c [k-1];
		 i--;
		 k--;
		 break;
	 case 1:
		 al_a += a [i-1];
		 al_b += '-';
		 al_c += '-';

		 i--;
		 break;
	 case 2:
		 al_a += '-';
		 al_b += '-';

		 al_c += c [k-1];

		 k--;
		 break;
	 case 11:
			 maxj=max_ind(s,i,k,b.length());

		     s_back(s,tb,a, b, c, al_a,al_b, al_c, match, unmatch , gap,maxj,i,k);
		     return;
	 case -2 :
		imax = max_2ind(s,i,b.length()).first;
		jmax = max_2ind(s,i,b.length()).second;
	    if (i>imax)
	    {
	    	count =0;
	    while(i>imax)
	    {
	    	 al_a += a [i -1];
			 al_b += '-';
			 al_c += '-';
			 count++;
			 i--;
	    }
	    }
	    
		 while(imax>0 && jmax>0)
		 {
//			 cout<<"tb"<<tb[imax][jmax][0]<<" "<<s[imax][jmax][0]<<endl;

			 switch (tb[imax][jmax][0])
			 {
			 case 0:
				 al_a += a [imax -1];
				 al_b += b [jmax -1];
				 al_c += '-';
//				 cout<<imax<<"."<<jmax<<" "<<s[imax][jmax][0]<<" ";
				 imax--;
				 jmax--;
//				 cout <<imax<<",0,"<<jmax<<endl;
			     break;
			 case 1:
				 al_a += a [imax -1];
				 al_b += '-';
				 al_c += '-';
//				 cout<<imax<<"."<<jmax<<" "<<s[imax][jmax][0]<<" ";
				 imax--;
//				 cout <<imax<<",1,"<<jmax<<endl;

				 break;
			 case 2:
				 al_a += '-';
				 al_b += b [jmax -1];
				 al_c += '-';
//				 cout<<imax<<"."<<jmax<<" "<<s[imax][jmax][0]<<" ";
				 jmax--;
//				 cout <<imax<<",2,"<<jmax<<endl;

				 break;
			 }
		 }
		 while(imax >0)
		 {
			 al_a += a [imax -1];
			 al_b += '-';
			 al_c += '-';
//			 cout<<imax<<"."<<jmax<<" "<<s[imax][jmax][0]<<" ";

			 imax--;

		 }
		 while(jmax >0)
		 {
			 al_a += '-';
			 al_b += b [jmax -1];
			 al_c += '-';
//			 cout<<imax<<"."<<jmax<<" "<<s[imax][jmax][0]<<" ";

			 jmax--;

		 }
		 return;


	 default:
		 cout<<endl<< "ERROR in 2d cases"<<endl;

	  }
	}

	while(i>0)
	{
		al_a+=a[i-1];
		al_b+='-';
		al_c+='-';

		i--;

	}
}

/**
 * intialization of main variables  and call of the algorithm functions.

 * @param a first sequence.
 * @param b second sequence.
 * @param c third sequence.
 * @param match match score.
 * @param unmatch mismatch score.
 * @param gap gap score.
 * @param sp sum of pairs weight.
 * @param outfile output file to where the alignment is written.
 */

 void test (string a, string b , string c, double match, double unmatch , double gap,float sp,string outfile,string format)
 {

     int m= a.length() , n = b.length() , p= c.length();
     string out;

     int ptr;
     string al_a, al_b, al_c;

        vector< vector< vector<float> > > s( m+1, vector<vector<float> >(n+1, vector<float>(p+1)) );
        vector< vector< vector <int> > > tb( m+1, vector<vector<int> >(n+1, vector<int>(p+1)) );
        vector <vector <float> > s2(m+1, vector<float>(p+1));
        vector<vector <int> >  tb2( m+1, vector<int> (p+1)) ;

         s_init(ptr,s,tb,m,n,p,a,b,c,match, unmatch, gap,sp);
         int maxj=-1;
         s_fill(ptr, s,tb,m,n,p,a,b,c, match, unmatch , gap,sp);
    	 s2_fill(ptr, s2, s,tb2,m,  n, p,a, b,  c, match,  unmatch ,  gap,  sp, maxj);
    	 s2_back(s,s2,tb2,tb,a, b, c, al_a,al_b, al_c, match, unmatch , gap,maxj);
         al_a = string (al_a.rbegin(), al_a.rend());
		 al_b = string (al_b.rbegin(), al_b.rend());
		 al_c = string (al_c.rbegin(), al_c.rend());
		 out =">F\n"+al_a+"\n>L\n"+al_b+"\n>R\n"+al_c;
		 write_output(outfile, out,format);

        }





 void usage(){
 	cout << "usage"<<endl;
 	
 	cout << "  call: ./brpali -i <input fasta file> -o <output file> -m <match score> -u <mismatch> -g <gap> -s <sum of pairs weight>"<<endl;
 	cout <<"options : -f format sto or fas default is fas"<<endl;
 	exit(1);
 }


 int  main(int argc, char *argv[] ){
		if (argc < 13) {
			cout <<argc<<endl ;
	 		cout << "Error in parameters count" << endl;
	 		
	 		for (int i =0; i<argc ; i++)
	 		{
	 			cout<<argv[i]<<endl;
	 		}
	 		usage();
	 	}

	 	int choice;
	 	float  unmatch= 0, match = 0, gap = 0,sp=0;
	 	string infile, outfile;
	 	string format = "";


	 	while ((choice = getopt(argc, argv, "m:u:g:s:i:o:f:")) != -1) {
	 		switch (choice) {


	 			case 'm':
	 				match = atof (optarg);
	 				break;
	 			case 'u':
	 				unmatch = atof(optarg);
	 				break;
	 			case 'g':
	 				gap = atof (optarg);
	 				break;
	 			case 's':
					sp = atof(optarg);
					break;
	 			case 'i':
	 				infile = optarg;
	 				break;
	 			case 'o':
	 				outfile = optarg;
	 				break;
	 			case 'f':
	 				format = optarg;
	 			

	 		}
	 	}


	 string a,b,c; //the three sequences
	 a = "";
	 b="";
	 c="";
	 read_infile(infile,a, b, c);

 (test(a,b,c , match, unmatch, gap,sp,outfile,format));


	 }





